% ------ Microchip classification ---------------------------------------------
% Logistic regression main script that loads test data and plots predictions
% Reference: github.com/trekhleb/machine-learning-octave
%  
clear;      %Clear all variables
close all;  %Close all other modules
clc;        %Clear the screen

% ------- Read training set----------------------------------------------------
fprintf('Loading the data...\n\n');

% microchips_tests.csv: training data set of microchip parameters and their validity
data = load('microchips_tests.csv');
X = data(:, 1:2); y = data(:, 3);

% Plotting the data -----------------------------------------------------------
fprintf('Plotting the data...\n\n');

% Find indices of positive [1] and negative [0] examples
positiveIndices = find(y == 1);  negativeIndices = find(y == 0);

% Plot examples.
hold on;
plot(X(positiveIndices, 1), X(positiveIndices, 2), 'k+', 'LineWidth', 2, 'MarkerSize', 7);
plot(X(negativeIndices, 1), X(negativeIndices, 2), 'ko', 'MarkerFaceColor', 'y', 'MarkerSize', 7);

% Draw labels and Legend
xlabel('Microchip Test 1');  ylabel('Microchip Test 2');
legend('y = 1', 'y = 0');

% Running logistic regression --------------------------------------------------
fprintf('Running logistic regression...\n\n');

% Add more polynomial features to make decision boundary to have complex curve form
polynomial_degree = 6;
X = add_polynomial_features(X(:, 1), X(:, 2), polynomial_degree);

% Run regression: logistic_regression_train.m = logistic regression algorithm
lambda = 1;
[theta, J, J_history, exit_flag] = logistic_regression_train(X, y, lambda);

fprintf('- Initial cost: %f\n', J_history(1));
fprintf('- Optimized cost: %f\n\n', J);

% Plotting decision boundaries -------------------------------------------------
fprintf('Plotting decision boundaries...\n\n');

% Generate a grid range.
u = linspace(-1, 1, 50);
v = linspace(-1, 1, 50);
z = zeros(length(u), length(v));
% Evaluate z = (x * theta) over the grid.
for i = 1:length(u)
  for j = 1:length(v)
    x = add_polynomial_features(u(i), v(j), polynomial_degree); % Add polinomials.
    x = [ones(size(x, 1), 1), x];     % Add ones.
    z(i, j) = x * theta;
  end
end

% It is mportant to transpose z before calling the contour.
z = z';

% Plot z = 0, notice you need to specify the range [0, 0]
contour(u, v, z, [0, 0], 'LineWidth', 2);
title(sprintf('lambda = %g', lambda));
legend('y = 1', 'y = 0', 'Decision boundary');

hold off;

% Trying to predict custom experiments ------------------------------------
fprintf('Trying to predict custom experiments...\n\n');

x = [0, 0; -0.5, -0.5];

% Add polynomials.
x = add_polynomial_features(x(:, 1), x(:, 2), polynomial_degree);
% Add ones.
x = [ones(size(x, 1), 1), x];

probabilities = hypothesis(x, theta);
fprintf(' %f \n', probabilities);